#!/usr/bin/python

"""
main.py - Main entry point for game
"""

import os
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from gamedirector3d import *

import resources
import game
import titlescenes

# Start
def main(mainpath,fullscreen=False):
	
	# Initialise pygame
	pygame.init()
	pygame.mixer.init()
	#pygame.mouse.set_visible(False)
	
	# start up director
	framerate = 30
	screen_res = (640,480)
	window_title = "Hackerbot"
	dir = GameDirector(window_title, screen_res, framerate, fullscreen)
	
	# Load resources
	resources.init(mainpath,screen_res)
	
	# Load titlescene, victoryscene
	titlescene = titlescenes.TitleScene(dir, screen_res)
	dir.addscene('titlescene', titlescene)
	victoryscene = titlescenes.VictoryScene(dir, screen_res)
	dir.addscene('victoryscene', victoryscene)
	
	# Load game scenes
	maingame = game.MainGame(dir, screen_res)
	maingame.progress_data.savepath = os.path.join(mainpath,'data','saved_progress.txt')
	dir.addscene('maingame', maingame)
	
	titlescene.h_maingame = maingame
	titlescene.h_progress_data = maingame.progress_data
	
	# start up director
	dir.change_scene('titlescene', [])
	#dir.change_scene('victoryscene', [])
	#dir.change_scene('maingame', [resources.MissionData['levels'][0]])
	dir.loop()
	
	# exiting, record framerate
	print "avg. frame rate: %f"%(maingame.avgframerate)



