#!/usr/bin/python

"""
resources.py - Load all resource data once 
"""

import os, sys
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

import numpy as np
from math import *

import objloader

class Model2(object):
	def __init__(self, filepath, swapyz=False, wrap_type='repeat'):
		self.obj = objloader.OBJ(filepath,swapyz=swapyz,wrap_type=wrap_type)

class Spritesheet(object):
	def __init__(self, path, sprite_size):
		self.size = sprite_size
		surf = pygame.image.load(path)
		w, h = surf.get_width(), surf.get_height()
		nc = w/self.size
		nr = h/self.size
		self.texture = []
		for j in xrange(nr):
			for i in xrange(nc):
				subsurf = pygame.Surface((self.size,self.size), SRCALPHA)
				subsurf.blit(surf, (0,0), area=(i*self.size, j*self.size, self.size, self.size))
				surf_data = pygame.image.tostring(subsurf, 'RGBA', 1)
				self.texture.append(glGenTextures(1))
				glBindTexture(GL_TEXTURE_2D, self.texture[-1])
				glPixelStorei(GL_UNPACK_ROW_LENGTH, 0)
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self.size, self.size, 0, GL_RGBA, GL_UNSIGNED_BYTE, surf_data)
	
	def render(self,x,y,s,ind):
		glEnable(GL_TEXTURE_2D)
		glBindTexture(GL_TEXTURE_2D, self.texture[ind])
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
		glBegin(GL_QUADS)
		glColor(1, 1, 1, 1)
		glTexCoord2f(0, 0)
		glVertex2f(x, y)
		glTexCoord2f(1, 0)
		glVertex2f(x+s*self.size, y)
		glTexCoord2f(1, 1)
		glVertex2f(x+s*self.size, y+s*self.size)
		glTexCoord2f(0, 1)
		glVertex2f(x, y+s*self.size)
		glEnd()
		glDisable(GL_TEXTURE_2D)

class DisplayText(object):
	def __init__(self, font, text, colour):
		surf = font.render(text, True, colour)
		self.w, self.h = surf.get_width(), surf.get_height()
		surf_data = pygame.image.tostring(surf, 'RGBA', 1)
		self.texture = glGenTextures(1)
		glBindTexture(GL_TEXTURE_2D, self.texture)
		glPixelStorei(GL_UNPACK_ROW_LENGTH, 0)
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self.w, self.h, 0, GL_RGBA, GL_UNSIGNED_BYTE, surf_data)
	
	def render(self,x,y,s,d):
		glEnable(GL_TEXTURE_2D)
		glBindTexture(GL_TEXTURE_2D, self.texture)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
		glBegin(GL_QUADS)
		glColor(1, 1, 1, 1)
		glTexCoord2f(0, 0)
		glVertex2f(x, y)
		glTexCoord2f(1, 0)
		glVertex2f(x+s*self.w, y)
		glTexCoord2f(1, 1)
		glVertex2f(x+s*self.w, y+s*self.h)
		glTexCoord2f(0, 1)
		glVertex2f(x, y+s*self.h)
		glEnd()
		glDisable(GL_TEXTURE_2D)

"""
class TextureSurf:
    def __init__(self, path):
    	surf = pygame.image.load(path)
        w, h = surf.get_width(), surf.get_height()
        self.width = w
        self.height = h
        surf_data = pygame.image.tostring(surf, 'RGBA', 1)
        self.texture = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, self.texture)
        glPixelStorei(GL_UNPACK_ROW_LENGTH, 0)
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, surf_data)
    
    def render(self):
    	glEnable(GL_TEXTURE_2D)
    	glBindTexture(GL_TEXTURE_2D, self.texture)
    	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glBegin(GL_QUADS)
        glColor(1, 1, 1, 1)
        glTexCoord2f(0, 0)
        glVertex2f(0, 0)
        glTexCoord2f(1, 0)
        glVertex2f(self.width, 0)
        glTexCoord2f(1, 1)
        glVertex2f(self.width, self.height)
        glTexCoord2f(0, 1)
        glVertex2f(0, self.height)
        glEnd()
        glDisable(GL_TEXTURE_2D)
"""

class MissionsParams(object):
	pass

def init(mainpath,screen_res):
    
    ######################
    # Mission Data
    
    global MissionData
    MissionData = {}
    
    MissionData['levels'] = ['tutorial1','tutorial2','tutorial3','level1','level2','level3','boss']
    #MissionData['levels'] = ['boss']    
    
    # Tutorial 1
    MissionData['tutorial1'] = MissionsParams()
    MissionData['tutorial1'].music = 'deceptive'
    MissionData['tutorial1'].map_limit = 250.0
    
    MissionData['tutorial1'].asteroids = []
    MissionData['tutorial1'].asteroids.append(['asteroid002atext2',[0.0,0.0,0.0]])
    MissionData['tutorial1'].asteroids.append(['asteroid002atext2',[50.0,-20.0,20.0]])

    MissionData['tutorial1'].shields = []
    MissionData['tutorial1'].hackerbotstart = [[0.0,200.0,0.0],[0.0,-1.0,0.0],-pi/2]
    #MissionData['tutorial'].hackerbotstart = [[0.0,50.0,0.0],[0.0,-1.0,0.0],-pi/2]
    
    MissionData['tutorial1'].npcs = []
    MissionData['tutorial1'].npcs.append(['datavault',1,3,None]) # ast num, orient type (0-5), None
    
    # Tutorial 2
    MissionData['tutorial2'] = MissionsParams()
    MissionData['tutorial2'].music = 'deceptive'
    MissionData['tutorial2'].map_limit = 400.0
    
    MissionData['tutorial2'].asteroids = []
    MissionData['tutorial2'].asteroids.append(['asteroid002atext2',[40.0,0.0,-20.0]])
    MissionData['tutorial2'].asteroids.append(['asteroid002atext2',[100.0,-50.0,-20.0]])
    
    MissionData['tutorial2'].shields = []
    MissionData['tutorial2'].shields.append([[100.0,-50.0,-20.0],25.0])
    MissionData['tutorial2'].hackerbotstart = [[40.0,200.0,-20.0],[0.0,-1.0,0.0],-pi/2]
    #MissionData['tutorial2'].hackerbotstart = [[0.0,50.0,0.0],[0.0,-1.0,0.0],-pi/2]
    
    MissionData['tutorial2'].npcs = []
    MissionData['tutorial2'].npcs.append(['shieldgen',0,0,0]) # ast num, vert num, shield ind
    MissionData['tutorial2'].npcs.append(['datavault',1,0,None]) # ast num, orient type (0-5), None
    
    # Tutorial 3
    MissionData['tutorial3'] = MissionsParams()
    MissionData['tutorial3'].music = 'deceptive'
    MissionData['tutorial3'].map_limit = 400.0
    
    MissionData['tutorial3'].asteroids = []
    MissionData['tutorial3'].asteroids.append(['asteroid002atext2',[90.0,-50.0, 0.0]])
    MissionData['tutorial3'].asteroids.append(['asteroid002atext2',[70.0,-150.0, 10.0]])
    
    MissionData['tutorial3'].asteroids.append(['asteroid002atext2',[90.0,-330.0, 0.0]])
    MissionData['tutorial3'].asteroids.append(['asteroid002atext2',[110.0,-260.0, 10.0]])
    MissionData['tutorial3'].asteroids.append(['asteroid002atext2',[20.0,-360.0, -40.0]])
    
    MissionData['tutorial3'].shields = []
    MissionData['tutorial3'].hackerbotstart = [[90.0,50.0,0.0],[0.0,-1.0,0.0],-pi/2]
    #MissionData['tutorial3'].hackerbotstart = [[0.0,50.0,0.0],[0.0,-1.0,0.0],-pi/2]
    
    MissionData['tutorial3'].npcs = []
    MissionData['tutorial3'].npcs.append(['datavault',1,5,None]) # ast num, orient type (0-5), None
    waypoints = [[90.0,-70.0,0.0],[60.0,-70.0,0.0],[70.0,-130.0,10.0],[80.0,-130.0,10.0]]
    MissionData['tutorial3'].npcs.append(['sentry',waypoints])
    MissionData['tutorial3'].npcs.append(['datavault',3,2,None]) # ast num, orient type (0-5), None
    MissionData['tutorial3'].npcs.append(['turret',2,3,None]) # ast num, orient type (0-5), None
    
    # Level 1
    MissionData['level1'] = MissionsParams()
    MissionData['level1'].music = 'deceptive'
    MissionData['level1'].map_limit = 400.0
    
    MissionData['level1'].asteroids = []
    MissionData['level1'].asteroids.append(['asteroid002atext2',[-30.0,0.0,0.0]])
    MissionData['level1'].asteroids.append(['asteroid002atext2',[0.0,-70.0,0.0]])
    MissionData['level1'].asteroids.append(['asteroid002atext2',[-100.0,-250.0,40.0]])
    MissionData['level1'].asteroids.append(['asteroid002atext2',[-50.0,-300.0,50.0]])
    MissionData['level1'].asteroids.append(['asteroid002atext2',[0.0,-300.0,50.0]])
    MissionData['level1'].asteroids.append(['asteroid002atext2',[-90.0,-100.0,40.0]])
    
    MissionData['level1'].shields = []
    MissionData['level1'].shields.append([[0.0,-300.0,50.0],25.0])
    MissionData['level1'].hackerbotstart = [[-30.0,100.0,0.0],[0.0,-1.0,0.0],-pi/2]
    
    MissionData['level1'].npcs = []
    MissionData['level1'].npcs.append(['datavault',1,1,None]) # ast num, orient type (0-5), None
    MissionData['level1'].npcs.append(['datavault',4,1,None]) # ast num, orient type (0-5), None
    
    waypoints = [[-30.0,-40.0,0.0],[-30.0,-100.0,0.0],[30.0,-100.0,0.0],[30.0,-40.0,0.0]]
    MissionData['level1'].npcs.append(['sentry',waypoints])
    waypoints = [[-125.0,-275.0,50.0],[-125.0,-330.0,50.0],[30.0,-330.0,50.0],[30.0,-275.0,50.0]]
    MissionData['level1'].npcs.append(['sentry',waypoints])
    waypoints = [[30.0,-330.0,50.0],[30.0,-275.0,50.0],[-125.0,-275.0,50.0],[-125.0,-330.0,50.0]]
    MissionData['level1'].npcs.append(['sentry',waypoints])
    MissionData['level1'].npcs.append(['shieldgen',3,3,0]) # ast num, orient type (0-5), None
    
    # Level 2
    MissionData['level2'] = MissionsParams()
    MissionData['level2'].music = 'deceptive'
    MissionData['level2'].map_limit = 400.0
    
    MissionData['level2'].asteroids = []
    MissionData['level2'].asteroids.append(['asteroid002atext2',[100.0,150.0,30.0]])
    MissionData['level2'].asteroids.append(['asteroid002atext2',[140.0,-50.0,10.0]])
    MissionData['level2'].asteroids.append(['asteroid002atext2',[75.0,-100.0,0.0]])
    MissionData['level2'].asteroids.append(['asteroid002atext2',[0.0,-50.0,0.0]])
    MissionData['level2'].asteroids.append(['asteroid002atext2',[0.0,0.0,0.0]])
    MissionData['level2'].asteroids.append(['asteroid002atext2',[-80.0,-5.0,50.0]])
    
    MissionData['level2'].shields = []
    MissionData['level2'].shields.append([[-80.0,-5.0,50.0],25.0])
    MissionData['level2'].hackerbotstart = [[100.0,250.0,30.0],[0.0,-1.0,0.0],-pi/2]
    
    MissionData['level2'].npcs = []
    MissionData['level2'].npcs.append(['datavault',1,1,None]) # ast num, orient type (0-5), None
    MissionData['level2'].npcs.append(['datavault',5,3,None]) # ast num, orient type (0-5), None
    waypoints = [[5.0,-90.0,0.0],[-5.0,-90.0,0.0],[0.0,-95.0,0.0]]
    MissionData['level2'].npcs.append(['sentry',waypoints])
    MissionData['level2'].npcs.append(['shieldgen',4,2,0]) # ast num, orient type (0-5), None
    MissionData['level2'].npcs.append(['turret',3,3,None]) # ast num, orient type (0-5), None
    
    # Level 3
    MissionData['level3'] = MissionsParams()
    MissionData['level3'].music = 'deceptive'
    MissionData['level3'].map_limit = 400.0
    
    MissionData['level3'].asteroids = []
    MissionData['level3'].asteroids.append(['asteroid002atext2',[0.0,0.0,-20.0]])
    MissionData['level3'].asteroids.append(['asteroid002atext2',[0.0,-50.0,-30.0]])
    MissionData['level3'].asteroids.append(['asteroid002atext2',[0.0,50.0,-30.0]])
    
    MissionData['level3'].asteroids.append(['asteroid002atext2',[60.0,200.0,10.0]])
    MissionData['level3'].asteroids.append(['asteroid002atext2',[-160.0,160.0,30.0]])
    MissionData['level3'].asteroids.append(['asteroid002atext2',[-200.0,-20.0,-20.0]])
    MissionData['level3'].asteroids.append(['asteroid002atext2',[-160.0,-160.0,5.0]])
    MissionData['level3'].asteroids.append(['asteroid002atext2',[60.0,-200.0,0.0]])
    
    MissionData['level3'].asteroids.append(['asteroid002atext2',[50.0,100.0,60.0]])
    MissionData['level3'].asteroids.append(['asteroid002atext2',[0.0,0.0,90.0]])
    
    MissionData['level3'].shields = []
    MissionData['level3'].shields.append([[0.0,0.0,90.0],25.0])
    MissionData['level3'].hackerbotstart = [[60.0,350.0,10.0],[0.0,-1.0,0.0],-pi/2]
    
    MissionData['level3'].npcs = []
    MissionData['level3'].npcs.append(['shieldgen',0,5,0]) # ast num, orient type (0-5), None
    MissionData['level3'].npcs.append(['turret',1,2,None]) # ast num, orient type (0-5), None
    MissionData['level3'].npcs.append(['turret',2,3,None]) # ast num, orient type (0-5), None
    MissionData['level3'].npcs.append(['datavault',9,1,None]) # ast num, orient type (0-5), None
    MissionData['level3'].npcs.append(['datavault',4,1,None]) # ast num, orient type (0-5), None
    MissionData['level3'].npcs.append(['datavault',7,2,None]) # ast num, orient type (0-5), None
    waypoints = [[-25.0,-25.0,-20.0],[-25.0,25.0,-20.0],[25.0,25.0,0.0],[25.0,-25.0,-20.0]]
    MissionData['level3'].npcs.append(['sentry',waypoints])
    waypoints = [[25.0,25.0,0.0],[25.0,-25.0,-20.0],[-25.0,-25.0,-20.0],[-25.0,25.0,-20.0]]
    MissionData['level3'].npcs.append(['sentry',waypoints])
    waypoints = [[60.0,170.0,10.0],[-180.0,140.0,30.0],[-170.0,-20.0,-20.0],[-130.0,-170.0,5.0],[80.0,-170.0,0.0]]
    MissionData['level3'].npcs.append(['sentry',waypoints])
    
    
    
    
    # Final Boss Stage
    MissionData['boss'] = MissionsParams()
    MissionData['boss'].music = 'volatile_reaction'
    MissionData['boss'].map_limit = 400.0
    
    MissionData['boss'].asteroids = []
    MissionData['boss'].asteroids.append(['asteroid002atext2',[0.0,0.0,50.0]])
    MissionData['boss'].asteroids.append(['asteroid002atext2little',[0.0,30.0,0.0]])
    MissionData['boss'].asteroids.append(['asteroid002atext2little',[0.0,-30.0,0.0]])
    
    MissionData['boss'].asteroids.append(['asteroid002atext2',[150.0,120.0,10.0]])
    MissionData['boss'].asteroids.append(['asteroid002atext2',[0.0,160.0,10.0]])
    MissionData['boss'].asteroids.append(['asteroid002atext2',[-155.0,125.0,-40.0]])
    MissionData['boss'].asteroids.append(['asteroid002atext2',[-155.0,-125.0,-20.0]])
    MissionData['boss'].asteroids.append(['asteroid002atext2',[0.0,-160.0,10.0]])
    MissionData['boss'].asteroids.append(['asteroid002atext2',[155.0,-125.0,-20.0]])
    
    MissionData['boss'].shields = []
    MissionData['boss'].shields.append([[0.0,0.0,0.0],25.0])
    #MissionData['boss'].shields.append([[0.0,0.0,100.0],25.0])
    MissionData['boss'].hackerbotstart = [[150.0,300.0,10.0],[0.0,-1.0,0.0],-pi/2]
    
    MissionData['boss'].npcs = []
    MissionData['boss'].npcs.append(['datavault',0,4,None]) # ast num, orient type (0-5), None
    MissionData['boss'].npcs.append(['shieldgen',1,1,0]) # ast num, vert num, shield ind
    MissionData['boss'].npcs.append(['shieldgen',2,1,0]) # ast num, vert num, shield ind
    MissionData['boss'].npcs.append(['juggernaut',0,0,0]) # ast num, vert num, shield ind
    
    ############
    # Models
    
    # skybox
    global model_skybox
    model_skybox = Model2(os.path.join(mainpath,'data','models','skybox','skybox.obj'),wrap_type='mirror')
    
    # Asteroids
    global model_asteroid
    model_asteroid = {}
    model_asteroid['asteroid002atext2'] = Model2(os.path.join(mainpath,'data','models','asteroid002atext2.obj'))
    model_asteroid['asteroid002atext2little'] = Model2(os.path.join(mainpath,'data','models','asteroid002atext2.obj'))
    model_asteroid['artificial'] = Model2(os.path.join(mainpath,'data','models','artificial.obj'))
    
    # hackerbot
    global model_hackerbot
    model_hackerbot = Model2(os.path.join(mainpath,'data','models','hackerbot_norm_colour3.obj'))
    global model_hackerbotlegfront
    global model_hackerbotlegback
    model_hackerbotlegback = Model2(os.path.join(mainpath,'data','models','hackerbot_leg2.obj'))
    model_hackerbotlegfront = Model2(os.path.join(mainpath,'data','models','hackerbot_leg3.obj'))
    
    
    # sentry
    global model_sentry
    model_sentry = Model2(os.path.join(mainpath,'data','models','sentry.obj'))
    
    # Shield
    global model_shield
    model_shield = Model2(os.path.join(mainpath,'data','models','shield.obj'))
    
    # plasma fire
    global model_plasma
    model_plasma = Model2(os.path.join(mainpath,'data','models','plasma.obj'))
    
    # missile
    global model_missile
    model_missile = Model2(os.path.join(mainpath,'data','models','missile.obj'))
    
    # static hackable models
    global model_shieldgen
    model_shieldgen = Model2(os.path.join(mainpath,'data','models','shieldgen.obj'))
    
    global model_datavault
    model_datavault = Model2(os.path.join(mainpath,'data','models','datavault.obj'))
    
    global model_turret
    model_turret = Model2(os.path.join(mainpath,'data','models','turret.obj'))
    
    global model_juggernaut
    model_juggernaut = Model2(os.path.join(mainpath,'data','models','juggernaut2.obj'))
    
    ###########################
    # Sprites and 2D Graphics
    
    # Fonts
    global font_huddata
    font_huddata = pygame.font.Font(os.path.join(mainpath,'data','fonts','kernel1.ttf'), 18)
    global font_huddata_big
    font_huddata_big = pygame.font.Font(os.path.join(mainpath,'data','fonts','kernel1.ttf'), 32)
    global font_huddata_bigger
    font_huddata_bigger = pygame.font.Font(os.path.join(mainpath,'data','fonts','kernel1.ttf'), 48)
    
    global title_sprites
    title_sprites = {}
    title_sprites['new_game_off'] = DisplayText(font_huddata_big, "New Game", (0, 204, 0))
    title_sprites['new_game_on'] = DisplayText(font_huddata_big, "New Game", (204, 204, 0))
    title_sprites['continue_off'] = DisplayText(font_huddata_big, "Continue", (0, 204, 0))
    title_sprites['continue_on'] = DisplayText(font_huddata_big, "Continue", (204, 204, 0))
    
    title_sprites['logo'] = DisplayText(font_huddata_bigger, "HackerBot", (0, 204, 0))
    title_sprites['victory'] = DisplayText(font_huddata_bigger, "Congratulations!", (0, 204, 0))
    
    title_sprites['victory1'] = DisplayText(font_huddata, "The HackerBot has served its masters well", (0, 204, 0))
    title_sprites['victory2'] = DisplayText(font_huddata, "HackerBot: a pyweek 24 entry by mit-mit", (0, 204, 0))
    title_sprites['victory3'] = DisplayText(font_huddata, "Thanks for playing!", (0, 204, 0))
        
    global HUD_surfs
    HUD_surfs = {}
    HUD_surfs['tohack'] = DisplayText(font_huddata, "Click and hold to hack target", (0, 204, 0))
    HUD_surfs['hacking'] = DisplayText(font_huddata, "Gaining access ...", (0, 204, 0))
    HUD_surfs['hackingdata'] = DisplayText(font_huddata, "Downloading data ...", (0, 204, 0))
    HUD_surfs['hackrange'] = DisplayText(font_huddata, "out of range", (204, 0, 0))
    HUD_surfs['canthack'] = DisplayText(font_huddata, "hack via turret generator", (204, 0, 0))
    
    HUD_surfs['shieldgen'] = DisplayText(font_huddata, "Shield Generator", (0, 204, 0))
    HUD_surfs['shieldgendis'] = DisplayText(font_huddata, "Shield Generator (Disabled)", (128, 128, 128))
    
    HUD_surfs['datavault'] = DisplayText(font_huddata, "Data Vault", (0, 204, 0))
    HUD_surfs['datavaultdis'] = DisplayText(font_huddata, "Data Vault (Hacked)", (128, 128, 128))
    
    HUD_surfs['sentry'] = DisplayText(font_huddata, "Sentrybot", (0, 204, 0))
    HUD_surfs['sentrydis'] = DisplayText(font_huddata, "Sentrybot (Disabled)", (128, 128, 128))
    
    HUD_surfs['turret'] = DisplayText(font_huddata, "Turret", (0, 204, 0))
    HUD_surfs['turretdis'] = DisplayText(font_huddata, "Turret (Disabled)", (128, 128, 128))
    
    HUD_surfs['juggernaut'] = DisplayText(font_huddata, "Juggernaut", (0, 204, 0))
    HUD_surfs['juggernautbig'] = DisplayText(font_huddata_big, "Juggernaut", (204, 204, 0))
    
    HUD_surfs['missile'] = DisplayText(font_huddata, "Missile", (0, 204, 0))
    
    HUD_surfs['paused'] = DisplayText(font_huddata_bigger, "Paused", (204, 204, 0))
    HUD_surfs['resume'] = DisplayText(font_huddata_big, "ESC to resume mission", (204, 204, 0))
    HUD_surfs['quit'] = DisplayText(font_huddata_big, "Q to Quit", (204, 204, 0))
    HUD_surfs['reset'] = DisplayText(font_huddata_big, "R to Reset Mission", (204, 204, 0))
    
    HUD_surfs['controls'] = DisplayText(font_huddata_big, "Controls:", (0, 204, 0))
    HUD_surfs['cont001'] = DisplayText(font_huddata, "Look around with mouse", (0, 204, 0))
    HUD_surfs['cont002'] = DisplayText(font_huddata, "A S W D: Move hackerbot around", (0, 204, 0))
    HUD_surfs['cont003'] = DisplayText(font_huddata, "Space: Jump between platforms", (0, 204, 0))
    HUD_surfs['cont004'] = DisplayText(font_huddata, "E: Toggle View (moving/hacking mode)", (0, 204, 0))
    HUD_surfs['cont005'] = DisplayText(font_huddata, "Left-click and hold: Hack target while in hack view mode", (0, 204, 0))
    
    HUD_surfs['shielddamaged'] = DisplayText(font_huddata, "Shield Damaged: other generators still active", (0, 204, 0))
    HUD_surfs['shielddisable'] = DisplayText(font_huddata, "Shield Disabled", (0, 204, 0))
    HUD_surfs['datavaulthack'] = DisplayText(font_huddata, "Data Vault Hacked!", (0, 204, 0))
    HUD_surfs['sentry_destroyed'] = DisplayText(font_huddata, "Sentrybot Destroyed", (0, 204, 0))
    HUD_surfs['sentry_disabled'] = DisplayText(font_huddata, "Sentrybot Disabled (30 sec)", (0, 204, 0))
    HUD_surfs['turret_disabled'] = DisplayText(font_huddata, "Turret Disabled", (0, 204, 0))
    HUD_surfs['missile_destroyed'] = DisplayText(font_huddata, "Missile Destroyed", (0, 204, 0))
    
    HUD_surfs['datavaulthackleft'] = []
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "Zero Data Vault Remaining ...", (0, 204, 0)))
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "One Data Vault Remaining ...", (0, 204, 0)))
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "Two Data Vaults Remaining ...", (0, 204, 0)))
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "Three Data Vaults Remaining ...", (0, 204, 0)))
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "Four Data Vaults Remaining ...", (0, 204, 0)))
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "Five Data Vaults Remaining ...", (0, 204, 0)))
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "Six Data Vaults Remaining ...", (0, 204, 0)))
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "Seven Data Vaults Remaining ...", (0, 204, 0)))
    HUD_surfs['datavaulthackleft'].append(DisplayText(font_huddata, "Eight Data Vaults Remaining ...", (0, 204, 0)))
    
    HUD_surfs['datavaulthackleft_red'] = []
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 0", (204, 0, 0)))
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 1", (204, 0, 0)))
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 2", (204, 0, 0)))
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 3", (204, 0, 0)))
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 4", (204, 0, 0)))
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 5", (204, 0, 0)))
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 6", (204, 0, 0)))
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 7", (204, 0, 0)))
    HUD_surfs['datavaulthackleft_red'].append(DisplayText(font_huddata, "Data Vault Remaining: 8", (204, 0, 0)))
    
    HUD_surfs['datavaulthackbegin'] = []
    HUD_surfs['datavaulthackbegin'].append(None)
    HUD_surfs['datavaulthackbegin'].append(DisplayText(font_huddata, "Data Vaults to access in this area: 1", (0, 204, 0)))
    HUD_surfs['datavaulthackbegin'].append(DisplayText(font_huddata, "Data Vaults to access in this area: 2", (0, 204, 0)))
    HUD_surfs['datavaulthackbegin'].append(DisplayText(font_huddata, "Data Vaults to access in this area: 3", (0, 204, 0)))
    HUD_surfs['datavaulthackbegin'].append(DisplayText(font_huddata, "Data Vaults to access in this area: 4", (0, 204, 0)))
    HUD_surfs['datavaulthackbegin'].append(DisplayText(font_huddata, "Data Vaults to access in this area: 5", (0, 204, 0)))
    HUD_surfs['datavaulthackbegin'].append(DisplayText(font_huddata, "Data Vaults to access in this area: 6", (0, 204, 0)))
    HUD_surfs['datavaulthackbegin'].append(DisplayText(font_huddata, "Data Vaults to access in this area: 7", (0, 204, 0)))
    HUD_surfs['datavaulthackbegin'].append(DisplayText(font_huddata, "Data Vaults to access in this area: 8", (0, 204, 0)))
    
    HUD_surfs['missioncomplete'] = DisplayText(font_huddata_big, "Mission Complete", (0, 204, 0))
    HUD_surfs['missionfailed'] = DisplayText(font_huddata_big, "Mission Failed", (204, 0, 0))
    
    HUD_surfs['death_destroy'] = DisplayText(font_huddata, "Hackerbot was destroyed", (204, 0, 0))
    HUD_surfs['death_shield'] = DisplayText(font_huddata, "Hackerbot ran into a shield", (204, 0, 0))
    HUD_surfs['death_float'] = DisplayText(font_huddata, "Hackerbot floated away", (204, 0, 0))
    
    HUD_surfs['targethacked'] = DisplayText(font_huddata, "Target Hacked: Control Gained", (204, 204, 0))
    HUD_surfs['control_release'] = DisplayText(font_huddata, "R: Release Target", (204, 204, 0))
    HUD_surfs['control_disable'] = DisplayText(font_huddata, "D: Disable Target (Temporary)", (204, 204, 0))
    HUD_surfs['control_take'] = DisplayText(font_huddata, "T: Take Control of Target", (204, 204, 0))
    HUD_surfs['control_fire'] = DisplayText(font_huddata, "SPACE/left-mouse: Fire Plasma Cannon", (204, 204, 0))
    
    HUD_surfs['activity'] = DisplayText(font_huddata, "Warning: Activity Noticed", (204, 204, 0))
    HUD_surfs['detected'] = DisplayText(font_huddata, "Warning: Presence Detected", (204, 0, 0))
    HUD_surfs['detectedmissile'] = DisplayText(font_huddata, "Warning: Incoming Missile", (204, 0, 0))
    
    HUD_surfs['revive'] = []
    for i in xrange(31):
    	HUD_surfs['revive'].append(DisplayText(font_huddata, "Time to revive: %d sec"%(i), (0, 204, 0)))
    
    HUD_surfs['tutorial'] = []
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Use A W S D keys to crawl around asteroids with Hackerbot", (0, 204, 0)))
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Look around with the mouse", (0, 204, 0)))
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Use the SPACE key to jump to other asteroids", (0, 204, 0)))
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Hackerbot will jump in the direction you are looking", (0, 204, 0)))
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Press E to toggle between hacking/moving mode", (0, 204, 0)))
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Left-click and hold to hack the data vault", (0, 204, 0)))
    
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Hack the shield generator to disable the nearby shield", (0, 204, 0)))
    
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Hack the Sentrybot to sneak past", (0, 204, 0)))
    
    HUD_surfs['tutorial'].append(DisplayText(font_huddata, "Watch out for turrets! Approach them from behind", (0, 204, 0)))
    
    global hud_target_sprites
    hud_target_sprites = Spritesheet(os.path.join(mainpath,'data','gfx','targets.png'), 32)
    
    global console_text_sprites
    console_text_sprites = {}
    console_text_sprites['access'] = Spritesheet(os.path.join(mainpath,'data','gfx','console_text1a.png'), 512)
    console_text_sprites['access2'] = Spritesheet(os.path.join(mainpath,'data','gfx','console_text1b.png'), 512)
    
    global explosion_sprites
    explosion_sprites = Spritesheet(os.path.join(mainpath,'data','gfx','explosion.png'), 128)
    
    #######################
    # Sound and Music
    
    # Sound Data
    global soundfx
    soundfx = {}
    
    soundfx['explode'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','explosion.ogg'))
    soundfx['laser'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','laser.ogg'))
    soundfx['hack'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','hack.ogg'))
    soundfx['alert'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','alert.ogg'))
    soundfx['check'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','check.ogg'))
    soundfx['success'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','success.ogg'))
    soundfx['missile'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','missile.ogg'))
    
    # Music Data
    global musicpaths
    musicpaths = {}
    musicpaths['title'] = os.path.join(mainpath,'data','music','phantom.ogg')
    musicpaths['deceptive'] = os.path.join(mainpath,'data','music','DST-Deceptive.ogg')
    musicpaths['volatile_reaction'] = os.path.join(mainpath,'data','music','volatile_reaction.ogg')
    
    # pre-sets and controls
    global controlmap
    controlmap = {}
    controlmap['R'] = K_d
    controlmap['L'] = K_a
    controlmap['D'] = K_s
    controlmap['U'] = K_w
    
    controlmap['View'] = K_e
    controlmap['Jump'] = K_SPACE
    controlmap['FreeFly'] = K_p

