#!/usr/bin/python

"""
world.py - classes for world, game environment, effects etc.
"""

import os
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from gamedirector3d import *
from math import *
import numpy as np

import resources

class Skybox(object):
	def __init__(self):
		pass
	
	def render(self):
		glLoadIdentity()
		glPushMatrix()
		glTranslate(*(self.h_motion.position[1], -self.h_motion.position[2], -self.h_motion.position[0]))
		glRotate(0.0, 1, 0, 0)
		glScale(1000.0, 1000.0, 1000.0)
		glCallList(resources.model_skybox.obj.gl_list)
		glPopMatrix()

class Shield(object):
	def __init__(self,parent,pos=[0.0,0.0,0.0],radius=25.0):
		self.parent = parent
		self.position = pos
		self.radius = radius
		self.rotation = [0.0,0.0,0.0]
		self.generators = []
		self.active=True
	
	def Update(self):
		if self.active:
			self.rotation[0] = (self.rotation[0] + 30) % 360 
			self.rotation[1] = (self.rotation[1] - 20) % 360 
			self.rotation[2] = (self.rotation[2] + 10) % 360
			dist2player = sqrt(pow(self.position[0]-self.h_hackerbot.position[0],2)+pow(self.position[1]-self.h_hackerbot.position[1],2)+pow(self.position[2]-self.h_hackerbot.position[2],2))
			if dist2player < self.radius and self.parent.playeralive == True:
				self.parent.playeralive = False
				self.parent.reasonfail = 'death_shield'
				self.parent.fadeto = 90
				self.h_explosions.New(self.h_hackerbot.position,1)
	
	def render(self):
		if self.active:
			glLoadIdentity()
			glPushMatrix()
		
			glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
			glRotate(self.rotation[0], 1, 0, 0)
			glRotate(self.rotation[1], 0, 1, 0)
			glRotate(self.rotation[2], 0, 0, 1)
		
			glScale(self.radius, self.radius, self.radius)
			glDisable(GL_DEPTH_TEST)
			glEnable(GL_BLEND)
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
			glColor(1, 1, 1, 0.5)
			glCallList(resources.model_shield.obj.gl_list)
			glDisable(GL_BLEND)
			glEnable(GL_DEPTH_TEST)
			glPopMatrix()

class Explosions(object):
	def __init__(self):
		self.n = 10
		self.position = []
		for i in xrange(self.n):
			self.position.append([0.0,0.0,0.0])
		self.position = np.array(self.position)
		self.to = [0]*self.n
		self.type = [0]*self.n
		self.scale = [1.0]*self.n
	
	def New(self,pos,type,scale=1.0):
		resources.soundfx['explode'].play()
		for i in xrange(self.n):
			if self.to[i] == 0:
				self.position[i,0] = pos[0]
				self.position[i,1] = pos[1]
				self.position[i,2] = pos[2]
				self.to[i] = 30
				self.type[i] = type
				self.scale[i] = scale
				break
	
	def Update(self):
		for i in xrange(self.n):
			if self.to[i] > 0:
				self.to[i] -= 1
				#if self.to[i] == 0:
				#	self.to[i] = 30
	
	def render(self):
		for i in xrange(self.n):
			if self.to[i] == 0:
				continue
			rel_pos = self.h_motion.position-self.position[i,:]
			azimuth = atan2(rel_pos[1],rel_pos[0])
			hordist = sqrt(rel_pos[0]*rel_pos[0] + rel_pos[1]*rel_pos[1])
			if hordist > 0.0:
				elev = -atan(rel_pos[2]/hordist)
			else:
				elev = pi/2
			
			glDisable(GL_LIGHTING)
			glLoadIdentity()
			glPushMatrix()
		
			glTranslate(*(self.position[i,1],-self.position[i,2],-self.position[i,0]))
			glRotate(-azimuth*180/pi, 0, 1, 0)
			glRotate(elev*180/pi, 1, 0, 0)
			size = 10.0
			glScale(self.scale[i], self.scale[i], self.scale[i])
			if self.type[i] == 1:
				glDisable(GL_DEPTH_TEST)
			glEnable(GL_BLEND)
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
			glEnable(GL_TEXTURE_2D)
			text_ind = 60-2*self.to[i]
			glBindTexture(GL_TEXTURE_2D, resources.explosion_sprites.texture[text_ind])
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
			glBegin(GL_QUADS)
			glColor(1, 1, 1, 1)
			glTexCoord2fv([0.0,0.0])
			glVertex3f(-size,-(-size),0.0)
			glTexCoord2fv([1.0,0.0])
			glVertex3f(size,-(-size),0.0)
			glTexCoord2fv([1.0,1.0])
			glVertex3f(size,-(size),0.0)
			glTexCoord2fv([0.0,1.0])
			glVertex3f(-size,-(size),0.0)
			glEnd()
			glDisable(GL_TEXTURE_2D)
			glDisable(GL_BLEND)
			if self.type[i] == 1:
				glEnable(GL_DEPTH_TEST)
			glPopMatrix()
			glEnable(GL_LIGHTING)

class Projectiles(object):
	def __init__(self,parent):
		self.parent = parent
		self.n = 10
		self.position = []
		self.velocity = []
		for i in xrange(self.n):
			self.position.append([0.0,0.0,0.0])
			self.velocity.append([0.0,0.0,0.0])
		self.position = np.array(self.position)
		self.velocity = np.array(self.velocity)
		self.to = [0]*self.n
		self.type = [0]*self.n
		self.firedby = [None]*self.n
	
	def New(self,pos,vel,type,firedby):
		resources.soundfx['laser'].play()
		for i in xrange(self.n):
			if self.to[i] == 0:
				self.position[i,0] = pos[0]
				self.position[i,1] = pos[1]
				self.position[i,2] = pos[2]
				self.velocity[i,0] = vel[0]
				self.velocity[i,1] = vel[1]
				self.velocity[i,2] = vel[2]
				self.to[i] = 120
				self.type[i] = type
				self.firedby[i] = firedby
				break
	
	def getprojectile_closestapproach(self, i, target_pos):
		posprev = self.position[i,:]-self.velocity[i,:]
		d2 = sqrt(np.sum((self.velocity[i,:])**2))
		d1 = sqrt(np.sum((np.cross(target_pos-posprev,target_pos-self.position[i,:]))**2))
		dist = d1/d2
		t = -np.dot((posprev-target_pos),self.velocity[i,:])/pow(d2,2)
		if t < 0:
			dist = sqrt(np.sum((target_pos-posprev)**2))
		elif t > 1.0:
			dist = sqrt(np.sum((target_pos-self.position[i,:])**2))
		return dist
	
	def Update(self):
		for i in xrange(self.n):
			if self.to[i] > 0:
				self.to[i] -= 1
				self.position[i,:] += self.velocity[i,:]
				distcheck = sqrt(np.sum((self.position[i,:]-self.h_hackerbot.position)**2))
				if self.parent.playeralive and not self.firedby[i] == self.h_hackerbot: # check for player hit
					dist = self.getprojectile_closestapproach(i, self.h_hackerbot.position)
					if dist <= self.h_hackerbot.radius and self.parent.playeralive and not self.parent.missioncomplete:
						self.parent.playeralive = False
						self.parent.reasonfail = 'death_destroy'
						self.parent.fadeto = 90
						self.parent.motion.mbdown = 0
						self.parent.motion.viewtype = 0
						self.h_hackerbot.hacking_state = 0
						self.h_explosions.New(self.h_hackerbot.position,1)
						self.to[i] = 0
						break
				if self.to[i] > 0:
					for shield in self.h_shields:
						dist = self.getprojectile_closestapproach(i, shield.position)
						if dist <= shield.radius and shield.active and not self.firedby[i] == self.parent.juggernaut:
							self.h_explosions.New(self.position[i,:]-self.velocity[i,:],0)
							self.to[i] = 0
							break
				if self.to[i] > 0:
					for npc in self.h_npcs:
						if not npc.state == 'dead' and not self.firedby[i] == npc:
							dist = self.getprojectile_closestapproach(i, npc.position)
							if dist <= npc.radius:
								npc.hit(0)
								self.parent.on_alert = True
								self.h_explosions.New(self.position[i,:],0,scale=2.0)
								self.to[i] = 0
								break
				if self.to[i] > 0:
					for asteroid in self.h_asteroids:
						dist = self.getprojectile_closestapproach(i, asteroid.position)
						if dist <= asteroid.radius:
							self.h_explosions.New(self.position[i,:]-self.velocity[i,:],0)
							self.to[i] = 0
							break
	
	def render(self):
		for i in xrange(self.n):
			if self.to[i] == 0:
				continue
			rel_pos = self.h_motion.position-self.position[i,:]
			azimuth = atan2(rel_pos[1],rel_pos[0])
			hordist = sqrt(rel_pos[0]*rel_pos[0] + rel_pos[1]*rel_pos[1])
			if hordist > 0.0:
				elev = -atan(rel_pos[2]/hordist)
			else:
				elev = pi/2
			
			glLoadIdentity()
			glPushMatrix()
		
			glTranslate(*(self.position[i,1],-self.position[i,2],-self.position[i,0]))
			glRotate(-azimuth*180/pi, 0, 1, 0)
			glRotate(elev*180/pi, 1, 0, 0)
			glScale(1.0, 1.0, 1.0)
			
			glDisable(GL_LIGHTING)
			glCallList(resources.model_plasma.obj.gl_list)
			glEnable(GL_LIGHTING)
			
			glPopMatrix()

class Asteroid(object):
	def __init__(self,type=1,id='asteroid002atext2',pos=[0.0,0.0,0.0]):
		self.type = type
		self.id = id
		self.position = pos
		self.rotation = [0.0,0.0,0.0]
		
		# grab model geometry, work out spatial information
		obj = resources.model_asteroid[self.id].obj
		
		self.centers = []
		self.radii = []
		for vert in obj.vertices:
			cx = vert[0]
			cy = vert[1]
			cz = vert[2]
			if self.id == 'asteroid002atext2little':
				cx = 0.3*cx
				cy = 0.3*cy
				cz = 0.3*cz
			self.centers.append([cz,-cx,cy])
			self.radii.append(pow(cx,2)+pow(cy,2)+pow(cz,2))
		self.centers = np.array(self.centers)
		
		i = np.argmin(self.centers[:,0])
		self.startpoint = self.centers[i,:]
		self.radius = sqrt(np.mean(self.radii))
	
	def render(self):
		
		glLoadIdentity()
		glPushMatrix()
		
		glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
		glRotate(-self.rotation[2]*180/pi, 0, 1, 0)
		glRotate(self.rotation[1]*180/pi, 1, 0, 0)
		
		if self.id == 'asteroid002atext2little':
			glScale(0.3, 0.3, 0.3)
		else:
			glScale(1.0, 1.0, 1.0)
		dist = sqrt(pow(self.position[0]-self.h_motion.position[0],2)+pow(self.position[1]-self.h_motion.position[1],2)+pow(self.position[2]-self.h_motion.position[2],2))
		dist -= 5.0
		if dist < self.radius:
			glDisable(GL_DEPTH_TEST)
			glEnable(GL_BLEND)
			#glBlendFunc(GL_ONE, GL_ONE)
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		glCallList(resources.model_asteroid[self.id].obj.gl_list)
		if dist < self.radius:
			glDisable(GL_BLEND)
			glEnable(GL_DEPTH_TEST)
		"""
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		glCallList(resources.model_asteroid[self.id].obj.gl_list)
		glDisable(GL_BLEND)
		"""
		glPopMatrix()




